"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Button = void 0;
const utils_js_1 = require("./utils.js");
class Button {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: 'button',
            buttonType: 'submit',
            className: 'btn form-button',
            template: 'Submit',
        };
        this.containerElement = null;
        this.buttonElement = null;
        this._isMounted = false;
        this._isVisible = true;
        this.initializeOptions(options);
        this._parent = parent;
        this._form = form;
        this._id = this.options.id;
        this._type = options.type;
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.reset();
        });
    }
    getId() {
        return this._id;
    }
    getContainer() {
        return this.containerElement;
    }
    getType() {
        return this._type;
    }
    getButtonType() {
        return this.options.buttonType;
    }
    getVisibility() {
        return this._isVisible;
    }
    createContainerElement() {
        this.containerElement = document.createElement('div');
        this.containerElement.className = 'form-button-container';
        this.containerElement.setAttribute('id', this._id + '_container');
    }
    createButtonElement() {
        this.buttonElement = document.createElement('button');
        this.buttonElement.className = this.options.className;
        this.buttonElement.setAttribute('id', this._id);
        this.buttonElement.setAttribute('type', this.options.buttonType);
        this.buttonElement.innerHTML = this.options.template;
        this.buttonElement.addEventListener('click', (event) => this.onClick(event));
    }
    onGui() {
        this.createContainerElement();
        this.createButtonElement();
        if (this.containerElement && this.buttonElement)
            (0, utils_js_1.mountElement)(this.buttonElement, this.containerElement);
    }
    mount() {
        if (this.buttonElement)
            (0, utils_js_1.mountElement)(this.buttonElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.buttonElement)
            (0, utils_js_1.unmountElement)(this.buttonElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            (0, utils_js_1.unmountElement)(this._parent);
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    onClick(event) {
        if (this.options.click)
            this.options.click(event, this._form.getData());
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateVisibilityBasedOnConditions();
            this.handleVisibility();
        });
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions)
            this._isVisible = this.options.conditions(this._form.getData());
    }
}
exports.Button = Button;
