"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RadioField = void 0;
const field_js_1 = require("../field.js");
const utils_js_1 = require("../utils.js");
class RadioField extends field_js_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: 'radio',
            schema: [],
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return 'This field is required';
                return true;
            },
            default: null,
            className: 'form-input-radio',
        };
        this.inputElements = [];
        this.labelElements = [];
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.handleVisibility();
        });
    }
    syncValue() {
        this.inputElements.forEach((el) => {
            if (el.value !== this.getValue() && el.hasAttribute('checked'))
                el.removeAttribute('checked');
            else if (el.value === this.getValue() && !el.hasAttribute('checked'))
                el.setAttribute('checked', '');
        });
    }
    createRadioLabelElement(labelText, id, input, container) {
        const labelElement = document.createElement('label');
        if (input && labelElement)
            (0, utils_js_1.mountElement)(input, labelElement);
        const label = document.createElement('p');
        if (this.options.label)
            label.innerText = labelText;
        label.setAttribute('id', id + '_label');
        labelElement.setAttribute('for', id);
        label.className = 'form-field-label';
        labelElement.append(label);
        this.labelElements.push(labelElement);
        if (container && labelElement)
            (0, utils_js_1.mountElement)(labelElement, container);
    }
    createRadioInputElement(id, value) {
        const inputElement = document.createElement('input');
        inputElement.setAttribute('id', id);
        inputElement.setAttribute('name', this.getId());
        inputElement.setAttribute('value', value);
        if (this.options.default && value === this.options.default)
            inputElement.setAttribute('checked', 'true');
        inputElement.setAttribute('type', this.getType());
        inputElement.className = this.options.className;
        this.inputElements.push(inputElement);
        this.bindChange();
        return inputElement;
    }
    onGui() {
        this.createContainerElement();
        const label = document.createElement('p');
        if (this.options.label)
            label.innerText = this.options.label;
        if (this.options.label && this.containerElement)
            (0, utils_js_1.mountElement)(label, this.containerElement);
        label.setAttribute('id', this.getId() + '_label');
        label.className = 'form-field-label';
        const container = document.createElement('div');
        container.className = 'form-field-container';
        this.options.schema.forEach((field) => {
            const input = this.createRadioInputElement(field.id, field.value);
            this.createRadioLabelElement(field.label, field.id, input, container);
        });
        if (this.containerElement && container)
            (0, utils_js_1.mountElement)(container, this.containerElement);
        this.createValidationElement();
        if (this.containerElement && this.validationElement)
            (0, utils_js_1.mountElement)(this.validationElement, this.containerElement);
    }
    handleValidatedField() {
        if (!this.getValidity()) {
            this.inputElements.forEach((el) => {
                el === null || el === void 0 ? void 0 : el.classList.add('form-error');
            });
            const vMessage = this.getValidationMessage();
            if (this.validationElement && vMessage) {
                this.validationElement.innerText = vMessage;
                this.validationElement.style.display = 'block';
            }
        }
        else {
            if (this.validationElement) {
                this.validationElement.style.display = 'none';
            }
            this.inputElements.forEach((el) => {
                el === null || el === void 0 ? void 0 : el.classList.remove('form-error');
            });
        }
        const form = this.getForm();
        form.updateError(this.getId(), this.getValidity());
    }
    bindChange() {
        if (this.inputElements[this.inputElements.length - 1])
            this.inputElements[this.inputElements.length - 1].addEventListener('change', (event) => {
                this.change(event);
            });
    }
}
exports.RadioField = RadioField;
