"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticField = void 0;
const utils_js_1 = require("../utils.js");
class StaticField {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: 'static',
            template: '',
            className: 'form-static',
        };
        this.staticElement = null;
        this.containerElement = null;
        this._isMounted = false;
        this._isVisible = true;
        this._value = null;
        this.initializeOptions(options);
        this._id = this.options.id;
        this._form = form;
        this._parent = parent;
        this._type = options.type;
        this._saveKey = (0, utils_js_1.generateFieldSaveKey)(this._form.getId(), this._id);
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
        });
    }
    getValue() {
        return this._value;
    }
    getId() {
        return this._id;
    }
    getSaveKey() {
        return this._saveKey;
    }
    getType() {
        return this._type;
    }
    getVisibility() {
        return this._isVisible;
    }
    getForm() {
        return this._form;
    }
    setTemplate(template, save = true) {
        if (this.staticElement)
            this.staticElement.innerHTML = template;
        if (save)
            this.save();
    }
    createContainerElement() {
        this.containerElement = document.createElement('div');
        this.containerElement.className = 'form-field ' + this._type;
        this.containerElement.setAttribute('id', this._id + '_container');
    }
    createStaticElement() {
        this.staticElement = document.createElement('div');
        this.staticElement.className = this.options.className;
        this.staticElement.setAttribute('id', this._id);
    }
    onGui() {
        this.createContainerElement();
        this.createStaticElement();
        if (this.containerElement && this.staticElement)
            (0, utils_js_1.mountElement)(this.staticElement, this.containerElement);
    }
    mount() {
        if (this.containerElement)
            (0, utils_js_1.mountElement)(this.containerElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.containerElement)
            (0, utils_js_1.unmountElement)(this.containerElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            (0, utils_js_1.unmountElement)(this._parent);
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    save() {
        if (this._form.savesProgress() && this._form.hasValidLicense() && this._value !== undefined) {
            localStorage.setItem(this._saveKey, JSON.stringify(this._value));
        }
    }
    load() {
        if (this._form.savesProgress() && this._form.hasValidLicense()) {
            const value = localStorage.getItem(this._saveKey);
            if (value !== null) {
                this.setTemplate(JSON.parse(value), false);
                return;
            }
        }
        this.setTemplate(this.options.template, false);
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.removeItem(this._saveKey);
            this.setTemplate(this.options.template, false);
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateVisibilityBasedOnConditions();
            this.handleVisibility();
        });
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions)
            this._isVisible = this.options.conditions(this._value, this._form.getData());
    }
}
exports.StaticField = StaticField;
