"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = void 0;
const constants_js_1 = require("./constants.js");
const utils_js_1 = require("./utils.js");
class Form {
    constructor(parentId, options) {
        this.options = {
            id: '',
            saveProgress: false,
            useFormData: false,
            schema: [],
            action: null,
            method: null,
            className: 'formjs-form',
        };
        this._parent = null;
        this._formElement = null;
        this._id = '';
        this._isValid = null;
        this._errors = [];
        this._fields = {};
        this._groups = {};
        this._buttons = {};
        this._data = {};
        this._saveProgress = false;
        this._licenseState = constants_js_1.LICENSE_STATE.INVALID;
        this._schema = [];
        this._dataPrefixMap = {};
        this.initializeOptions(options);
        this.setParentElement(parentId);
        this.processLicense();
        this.onGui();
        this.initForm();
    }
    setParentElement(parentId) {
        this._parent = document.getElementById(parentId);
        if (!this._parent)
            throw new Error('Parent does not exist in the document');
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
        this._id = options.id;
        this._schema = options.schema;
        this._saveProgress = options.saveProgress;
    }
    initForm() {
        if (this._formElement)
            this.buildSchema(this._schema, this._formElement);
        if (this._formElement && this._parent)
            (0, utils_js_1.mountElement)(this._formElement, this._parent);
    }
    processLicense() {
        if (!(0, utils_js_1.usesLicensedFetures)())
            return;
        if (this.options.licenseKey)
            (0, utils_js_1.setLicenseKey)(this.options.licenseKey);
        this._licenseState = (0, utils_js_1.processLicenseKey)();
        if (!this.hasValidLicense()) {
            (0, utils_js_1.handleInvalidLicenseLog)(this._licenseState);
        }
    }
    buildSchema(schema, parent, groupId = null, listId = null, key = null) {
        schema.forEach((options) => {
            this.buildElement(options, parent, groupId, listId, key);
        });
    }
    isFormValid() {
        return this._isValid;
    }
    createWrapper(parent) {
        const wrapper = document.createElement('div');
        (0, utils_js_1.mountElement)(wrapper, parent);
        return wrapper;
    }
    mapFieldToDataPrefix(options, id, key = null) {
        let newId = options.id;
        if (key) {
            newId = (0, utils_js_1.transformFieldName)(id, key, options.id);
        }
        this._dataPrefixMap[newId] = {
            id: id,
            dataKey: options.id,
            key: key,
        };
        if (key) {
            options.id = newId;
            options.name = newId;
        }
    }
    buildElement(options, parent, groupId, listId, key) {
        const duplicatedOptions = Object.assign({}, options);
        const Constructor = constants_js_1.elementConstructors[options.type];
        if (!Constructor)
            throw new Error(`Unknown type: ${options.type}`);
        const wrapper = this.createWrapper(parent);
        const Constructed = new Constructor(wrapper, this, duplicatedOptions);
        const formElementType = (0, utils_js_1.getFormElementType)(options.type);
        switch (formElementType) {
            case constants_js_1.costructorTypes.button:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._buttons[options.id] = Constructed;
                break;
            case constants_js_1.costructorTypes.group:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._groups[options.id] = Constructed;
                if (options.schema && Constructed.getSchemaContainer) {
                    const newParent = Constructed.getSchemaContainer();
                    if (newParent) {
                        this.buildSchema(options.schema, newParent, options.prefixSchema ? Constructed.getId() : groupId, listId, key);
                    }
                }
                break;
            case constants_js_1.costructorTypes.field:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._fields[options.id] = Constructed;
                break;
        }
    }
    assignToListField(listId, key, constructed, formElementType, id) {
        const list = this._fields[listId];
        if (!list)
            return;
        switch (formElementType) {
            case constants_js_1.costructorTypes.button:
                list.assignButton(id, key, constructed);
                break;
            case constants_js_1.costructorTypes.group:
                list.assignGroup(id, key, constructed);
                break;
            case constants_js_1.costructorTypes.field:
                list.assignField(id, key, constructed);
                break;
        }
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            Object.keys(this._groups).forEach((key) => {
                this._groups[key].update();
            });
            Object.keys(this._fields).forEach((key) => {
                this._fields[key].update();
            });
            Object.keys(this._buttons).forEach((key) => {
                this._buttons[key].update();
            });
        });
    }
    getField(id) {
        return this._fields[id];
    }
    getFormElement() {
        return this._formElement;
    }
    setData(id, value) {
        if (this._dataPrefixMap.hasOwnProperty(id)) {
            this.setDataFromMap(id, value);
        }
        else {
            this.setSimpleData(id, value);
        }
        this.update();
    }
    setDataFromMap(id, value) {
        const fieldMapping = this._dataPrefixMap[id];
        const { dataKey, key, id: prefixDataId } = fieldMapping;
        if (!dataKey || !prefixDataId)
            return;
        if (key) {
            const list = this.getField(prefixDataId);
            if (!list)
                return;
            const prefixId = list.getId();
            const keyIndex = list.getKeyIndex(key);
            this.ensureDataStructureExists(prefixId, keyIndex);
            this._data[prefixId][keyIndex][dataKey] = value;
        }
        else {
            const group = this.getGroup(prefixDataId);
            if (!group)
                return;
            const prefixId = group.getId();
            this.ensureDataStructureExists(prefixId);
            this._data[prefixId][dataKey] = value;
        }
    }
    setSimpleData(id, value) {
        this._data[id] = value;
    }
    ensureDataStructureExists(listId, keyIndex = null) {
        if (keyIndex === null && !this._data[listId]) {
            this._data[listId] = {};
            return;
        }
        else if (keyIndex || keyIndex === 0) {
            if (!this._data[listId])
                this._data[listId] = [];
            if (!this._data[listId][keyIndex])
                this._data[listId][keyIndex] = {};
        }
    }
    removeListData(listId, index) {
        if (this._data[listId] && this._data[listId][index]) {
            this._data[listId].splice(index, 1);
        }
    }
    getId() {
        return this._id;
    }
    getData() {
        return this._data;
    }
    getButtons() {
        return this._buttons;
    }
    getButton(id) {
        return this._buttons[id];
    }
    getGroups() {
        return this._groups;
    }
    getGroup(id) {
        return this._groups[id];
    }
    savesProgress() {
        return this._saveProgress;
    }
    hasValidLicense() {
        return this._licenseState === constants_js_1.LICENSE_STATE.VALID;
    }
    updateError(id, isValid) {
        if (this._errors.includes(id) && isValid) {
            this._errors = this._errors.filter((field) => {
                return field !== id;
            });
        }
        else if (!this._errors.includes(id) && !isValid) {
            this._errors.push(id);
        }
        this._isValid = this._errors.length === 0;
    }
    reset(event) {
        if (event)
            event.preventDefault();
        Object.keys(this._groups).forEach((key) => {
            this._groups[key].reset();
        });
        Object.keys(this._fields).forEach((key) => {
            this._fields[key].reset();
        });
        Object.keys(this._buttons).forEach((key) => {
            this._buttons[key].reset();
        });
    }
    validate() {
        Object.keys(this._fields).forEach((key) => {
            if (this._fields[key].validate)
                this._fields[key].validate();
        });
    }
    save() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            this._fields[key].save();
        });
        Object.keys(this._groups).forEach((key) => {
            if (typeof this._groups[key].save === 'function')
                this._groups[key].save();
        });
    }
    load() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            this._fields[key].load();
        });
        Object.keys(this._groups).forEach((key) => {
            if (typeof this._groups[key].save === 'function')
                this._groups[key].load();
        });
    }
    onGui() {
        this._formElement = document.createElement('form');
        this._formElement.setAttribute('id', this._id);
        this._formElement.className = this.options.className;
        if (this.options.action)
            this._formElement.setAttribute('action', this.options.action);
        if (this.options.method)
            this._formElement.setAttribute('method', this.options.method);
        this._formElement.addEventListener('submit', (event) => this.submit(event, this));
        this._formElement.addEventListener('reset', this.reset);
        if ((0, utils_js_1.usesLicensedFetures)() && !this.hasValidLicense())
            this.createInvalidElement();
    }
    createInvalidElement() {
        const invalid = document.createElement('a');
        invalid.setAttribute('href', constants_js_1.PACKAGE_LICENSE_URL);
        invalid.className = constants_js_1.licensePlateClass;
        invalid.style.cssText = constants_js_1.licensePlateStyle;
        const text = document.createElement('span');
        text.innerText = (0, utils_js_1.getLicenseText)(this._licenseState);
        (0, utils_js_1.mountElement)(text, invalid);
        if (this._parent)
            (0, utils_js_1.mountElement)(invalid, this._parent);
    }
    submit(event, form) {
        event.preventDefault();
        form.validate();
        if (!form.isFormValid()) {
            return;
        }
        if (this.options.action) {
            const element = this.getFormElement();
            if (element)
                element.submit();
        }
        let data = form.getData();
        if (this.options.useFormData) {
            data = (0, utils_js_1.objectToFormData)(form.getData());
        }
        if (form.options.submit)
            form.options.submit(data);
    }
}
exports.Form = Form;
