"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Row = void 0;
const utils_js_1 = require("./utils.js");
class Row {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: 'row',
            className: 'form-row',
            schema: [],
        };
        this.containerElement = null;
        this.labelElement = null;
        this.rowElement = null;
        this._isMounted = false;
        this._isVisible = true;
        this.initializeOptions(options);
        this._parent = parent;
        this._form = form;
        this._id = this.options.id;
        this._type = options.type;
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.reset();
        });
    }
    getId() {
        return this._id;
    }
    getContainer() {
        return this.containerElement;
    }
    getRow() {
        return this.rowElement;
    }
    getType() {
        return this._type;
    }
    getVisibility() {
        return this._isVisible;
    }
    createContainerElement() {
        this.containerElement = document.createElement('div');
        this.containerElement.className = 'form-row-container';
        this.containerElement.setAttribute('id', this._id + '_container');
    }
    createRowElement() {
        this.rowElement = document.createElement('div');
        this.rowElement.className = this.options.className;
        this.rowElement.setAttribute('id', this._id);
    }
    createLabelElement() {
        this.labelElement = document.createElement('p');
        if (this.options.label)
            this.labelElement.innerText = this.options.label;
        this.labelElement.setAttribute('id', this._id + '_label');
        this.labelElement.className = 'form-row-label';
    }
    onGui() {
        this.createContainerElement();
        this.createLabelElement();
        this.createRowElement();
        if (this.containerElement && this.labelElement && this.options.label)
            (0, utils_js_1.mountElement)(this.labelElement, this.containerElement);
        if (this.containerElement && this.rowElement)
            (0, utils_js_1.mountElement)(this.rowElement, this.containerElement);
    }
    mount() {
        if (this.containerElement)
            (0, utils_js_1.mountElement)(this.containerElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.containerElement)
            (0, utils_js_1.unmountElement)(this.containerElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            (0, utils_js_1.unmountElement)(this._parent);
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateVisibilityBasedOnConditions();
            this.handleVisibility();
        });
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions)
            this._isVisible = this.options.conditions(this._form.getData());
    }
}
exports.Row = Row;
