"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLicensedFetures = exports.usesLicensedFetures = exports.handleInvalidLicenseLog = exports.getLicenseText = exports.setLicenseKey = exports.processLicenseKey = exports.generateFieldSaveKey = exports.transformFieldName = exports.createWrapper = exports.extractFieldsFromSchema = exports.isListField = exports.getFormElementType = exports.isButton = exports.isGroup = exports.isField = exports.objectToFormData = exports.unmountElement = exports.mountElement = exports.debounce = void 0;
const constants_1 = require("./constants");
let LICENSE_KEY = null;
let USES_LICENSED_FETURES = false;
const debounce = (callback, wait, context) => {
    let timeoutId = null;
    return (...args) => {
        if (timeoutId)
            window.clearTimeout(timeoutId);
        timeoutId = window.setTimeout(() => {
            callback.apply(context, args);
        }, wait);
    };
};
exports.debounce = debounce;
const mountElement = (element, parent) => {
    parent.append(element);
};
exports.mountElement = mountElement;
const unmountElement = (element) => {
    var _a;
    (_a = element.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(element);
};
exports.unmountElement = unmountElement;
const objectToFormData = (obj, form, namespace) => {
    const fd = form || new FormData();
    let formKey;
    for (const property in obj) {
        if (obj.hasOwnProperty(property)) {
            if (namespace) {
                formKey = namespace + '[' + property + ']';
            }
            else {
                formKey = property;
            }
            if (obj[property] == null) {
                fd.append(formKey, '');
            }
            else if (typeof obj[property] === 'object' && !(obj[property] instanceof File)) {
                (0, exports.objectToFormData)(obj[property], fd, property);
            }
            else {
                fd.append(formKey, obj[property]);
            }
        }
    }
    return fd;
};
exports.objectToFormData = objectToFormData;
const isField = (type) => {
    return constants_1.fields.includes(type);
};
exports.isField = isField;
const isGroup = (type) => {
    return constants_1.groups.includes(type);
};
exports.isGroup = isGroup;
const isButton = (type) => {
    return constants_1.buttons.includes(type);
};
exports.isButton = isButton;
const getFormElementType = (type) => {
    if ((0, exports.isField)(type))
        return constants_1.costructorTypes.field;
    if ((0, exports.isGroup)(type))
        return constants_1.costructorTypes.group;
    if ((0, exports.isButton)(type))
        return constants_1.costructorTypes.button;
    return null;
};
exports.getFormElementType = getFormElementType;
const isListField = (type) => {
    return (0, exports.isField)(type) && type === 'list';
};
exports.isListField = isListField;
const extractFieldsFromSchema = (schema) => {
    let fieldsList = [];
    schema.forEach((options) => {
        if (constants_1.fields.includes(options.type))
            fieldsList.push(options.id);
        else if (options.schema)
            fieldsList = [...constants_1.fields, ...(0, exports.extractFieldsFromSchema)(options.schema)];
    });
    return fieldsList;
};
exports.extractFieldsFromSchema = extractFieldsFromSchema;
const createWrapper = (parent) => {
    const wrapper = document.createElement('div');
    (0, exports.mountElement)(wrapper, parent);
    return wrapper;
};
exports.createWrapper = createWrapper;
const transformFieldName = (parentId, key, id) => {
    return parentId + '[' + key + ']' + id;
};
exports.transformFieldName = transformFieldName;
const generateFieldSaveKey = (formId, id) => {
    return '__formsjs_' + formId + '_field_' + id;
};
exports.generateFieldSaveKey = generateFieldSaveKey;
const isDateValid = (d) => d instanceof Date && !isNaN(d);
const calculateDateAfterAddingDays = (date, days) => {
    const result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
};
const parseKey = (key) => key.match(/^(\d+)@(\d+)$/);
const isKeyInLicenseKeys = (key) => constants_1.OS_LICENSE_KEYS.includes(key);
const calculateMinPurchaseDate = (releaseDate) => calculateDateAfterAddingDays(releaseDate, -constants_1.UPGRADE_WINDOW);
const determineLicenseState = (purchaseDate, minPurchaseDate) => minPurchaseDate < purchaseDate ? constants_1.LICENSE_STATE.VALID : constants_1.LICENSE_STATE.OUTDATED;
const processLicenseKey = () => {
    var _a;
    const key = LICENSE_KEY;
    if (!key)
        return constants_1.LICENSE_STATE.INVALID;
    if (isKeyInLicenseKeys(key))
        return constants_1.LICENSE_STATE.VALID;
    const parts = parseKey(key);
    if (((_a = parts === null || parts === void 0 ? void 0 : parts[1]) === null || _a === void 0 ? void 0 : _a.length) === 10) {
        const [, , timestamp] = parts;
        const purchaseDate = new Date(parseInt(timestamp, 10) * 1000);
        const releaseDate = new Date(constants_1.RELEASE_DATE);
        if (isDateValid(releaseDate)) {
            const minPurchaseDate = calculateMinPurchaseDate(releaseDate);
            return determineLicenseState(purchaseDate, minPurchaseDate);
        }
    }
    return constants_1.LICENSE_STATE.INVALID;
};
exports.processLicenseKey = processLicenseKey;
const setLicenseKey = (key) => {
    LICENSE_KEY = key;
};
exports.setLicenseKey = setLicenseKey;
const getLicenseText = (license) => {
    if (license === constants_1.LICENSE_STATE.VALID)
        return 'forms.js license key is valid';
    if (license === constants_1.LICENSE_STATE.OUTDATED)
        return 'forms.js license key is outdated';
    return 'forms.js license key is invalid';
};
exports.getLicenseText = getLicenseText;
const handleInvalidLicenseLog = (license) => {
    console.error('***********************************************');
    if (license === constants_1.LICENSE_STATE.INVALID) {
        console.error('forms.js license key is invalid, please use a valid license key');
    }
    else {
        console.error('forms.js license key is outdated, please update your license key');
    }
    console.error('***********************************************');
};
exports.handleInvalidLicenseLog = handleInvalidLicenseLog;
const usesLicensedFetures = () => {
    return USES_LICENSED_FETURES;
};
exports.usesLicensedFetures = usesLicensedFetures;
const useLicensedFetures = () => {
    USES_LICENSED_FETURES = true;
};
exports.useLicensedFetures = useLicensedFetures;
