var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as TomSelectNamespace from 'tom-select';
const TomSelect = TomSelectNamespace.default;
import { Field } from '../field.js';
import { debounce, mountElement } from '../utils.js';
export class SelectField extends Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: 'select',
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return 'This field is required';
                return true;
            },
            enhance: true,
            multiple: false,
            optionsList: [],
            className: 'form-input-select',
            options: {},
        };
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.initTomselect();
            this.load();
            this.update();
            this.bindChange();
        });
    }
    syncValue() {
        if (this.inputElement && this.inputElement instanceof HTMLSelectElement && this.options.enhance === false) {
            const value = this.getValue();
            for (let option of this.inputElement.options) {
                const hasValue = value && Array.isArray(value) ? value === null || value === void 0 ? void 0 : value.some((val) => val === option.value) : value === option.value;
                if (hasValue && !option.hasAttribute('selected'))
                    option.setAttribute('selected', '');
                if (!hasValue && option.hasAttribute('selected'))
                    option.removeAttribute('selected');
            }
        }
        else if (this.options.enhance === true && this._tomselect) {
            const value = this.getValue();
            if (value !== this._tomselect.getValue())
                this._tomselect.setValue(value, true);
        }
    }
    bindChange() {
        if (this.inputElement)
            this.inputElement.addEventListener('change', debounce(this.change, 25, this));
    }
    getTomselect() {
        return this._tomselect;
    }
    initTomselect() {
        if (this.inputElement && this.options.enhance)
            this._tomselect = new TomSelect(this.inputElement, this.options.options || {});
    }
    handleDisabled() {
        var _a, _b;
        if (this.isDisabled()) {
            (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.setAttribute('disabled', 'true');
            if (this._tomselect)
                this._tomselect.disable();
        }
        else {
            (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.removeAttribute('disabled');
            if (this._tomselect)
                this._tomselect.enable();
        }
    }
    createSelectElement() {
        var _a;
        this.inputElement = document.createElement('select');
        this.inputElement.setAttribute('id', this.getId());
        this.inputElement.setAttribute('name', this.options.name || this.getId());
        this.inputElement.setAttribute('type', this.getType());
        if (this.options.multiple)
            this.inputElement.setAttribute('multiple', '');
        this.inputElement.className = this.options.className;
        (_a = this.options.optionsList) === null || _a === void 0 ? void 0 : _a.forEach((option) => {
            var _a, _b;
            const optionElement = document.createElement('option');
            optionElement.setAttribute('value', option.value);
            if (typeof option.value === 'string' &&
                this.options.default &&
                Array.isArray(this.options.default) &&
                ((_a = this.options.default) === null || _a === void 0 ? void 0 : _a.findIndex((val) => val === option.value)) >= 0) {
                optionElement.setAttribute('selected', 'true');
            }
            else if (this.options.default && this.options.default === option.value) {
                optionElement.setAttribute('selected', 'true');
            }
            if (option.disabled)
                optionElement.setAttribute('disabled', String(option.disabled));
            optionElement.innerText = option.label;
            (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.append(optionElement);
        });
    }
    onGui() {
        this.createContainerElement();
        this.createSelectElement();
        this.createLabelElement();
        this.createValidationElement();
        if (this.inputElement && this.labelElement)
            mountElement(this.inputElement, this.labelElement);
        if (this.labelElement && this.validationElement)
            mountElement(this.validationElement, this.labelElement);
        if (this.containerElement && this.labelElement)
            mountElement(this.labelElement, this.containerElement);
    }
    change(event) {
        if (this.options.enhance && this._tomselect) {
            this.setValue(this._tomselect.getValue());
        }
        else if (this.inputElement && this.inputElement instanceof HTMLSelectElement) {
            if (this.options.multiple) {
                const options = this.inputElement.options;
                const selectedValues = [];
                for (let i = 0; i < options.length; i++) {
                    if (options[i].selected) {
                        selectedValues.push(options[i].value);
                    }
                }
                this.setValue(selectedValues);
            }
            else {
                this.setValue(event.target.value);
            }
        }
        this.validate();
        if (this.options.change)
            this.options.change(this.getValue());
    }
}
