import { LICENSE_STATE, OS_LICENSE_KEYS, RELEASE_DATE, UPGRADE_WINDOW, buttons, costructorTypes, fields, groups, } from './constants';
let LICENSE_KEY = null;
let USES_LICENSED_FETURES = false;
export const debounce = (callback, wait, context) => {
    let timeoutId = null;
    return (...args) => {
        if (timeoutId)
            window.clearTimeout(timeoutId);
        timeoutId = window.setTimeout(() => {
            callback.apply(context, args);
        }, wait);
    };
};
export const mountElement = (element, parent) => {
    parent.append(element);
};
export const unmountElement = (element) => {
    var _a;
    (_a = element.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(element);
};
export const objectToFormData = (obj, form, namespace) => {
    const fd = form || new FormData();
    let formKey;
    for (const property in obj) {
        if (obj.hasOwnProperty(property)) {
            if (namespace) {
                formKey = namespace + '[' + property + ']';
            }
            else {
                formKey = property;
            }
            if (obj[property] == null) {
                fd.append(formKey, '');
            }
            else if (typeof obj[property] === 'object' && !(obj[property] instanceof File)) {
                objectToFormData(obj[property], fd, property);
            }
            else {
                fd.append(formKey, obj[property]);
            }
        }
    }
    return fd;
};
export const isField = (type) => {
    return fields.includes(type);
};
export const isGroup = (type) => {
    return groups.includes(type);
};
export const isButton = (type) => {
    return buttons.includes(type);
};
export const getFormElementType = (type) => {
    if (isField(type))
        return costructorTypes.field;
    if (isGroup(type))
        return costructorTypes.group;
    if (isButton(type))
        return costructorTypes.button;
    return null;
};
export const isListField = (type) => {
    return isField(type) && type === 'list';
};
export const extractFieldsFromSchema = (schema) => {
    let fieldsList = [];
    schema.forEach((options) => {
        if (fields.includes(options.type))
            fieldsList.push(options.id);
        else if (options.schema)
            fieldsList = [...fields, ...extractFieldsFromSchema(options.schema)];
    });
    return fieldsList;
};
export const createWrapper = (parent) => {
    const wrapper = document.createElement('div');
    mountElement(wrapper, parent);
    return wrapper;
};
export const transformFieldName = (parentId, key, id) => {
    return parentId + '[' + key + ']' + id;
};
export const generateFieldSaveKey = (formId, id) => {
    return '__formsjs_' + formId + '_field_' + id;
};
const isDateValid = (d) => d instanceof Date && !isNaN(d);
const calculateDateAfterAddingDays = (date, days) => {
    const result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
};
const parseKey = (key) => key.match(/^(\d+)@(\d+)$/);
const isKeyInLicenseKeys = (key) => OS_LICENSE_KEYS.includes(key);
const calculateMinPurchaseDate = (releaseDate) => calculateDateAfterAddingDays(releaseDate, -UPGRADE_WINDOW);
const determineLicenseState = (purchaseDate, minPurchaseDate) => minPurchaseDate < purchaseDate ? LICENSE_STATE.VALID : LICENSE_STATE.OUTDATED;
export const processLicenseKey = () => {
    var _a;
    const key = LICENSE_KEY;
    if (!key)
        return LICENSE_STATE.INVALID;
    if (isKeyInLicenseKeys(key))
        return LICENSE_STATE.VALID;
    const parts = parseKey(key);
    if (((_a = parts === null || parts === void 0 ? void 0 : parts[1]) === null || _a === void 0 ? void 0 : _a.length) === 10) {
        const [, , timestamp] = parts;
        const purchaseDate = new Date(parseInt(timestamp, 10) * 1000);
        const releaseDate = new Date(RELEASE_DATE);
        if (isDateValid(releaseDate)) {
            const minPurchaseDate = calculateMinPurchaseDate(releaseDate);
            return determineLicenseState(purchaseDate, minPurchaseDate);
        }
    }
    return LICENSE_STATE.INVALID;
};
export const setLicenseKey = (key) => {
    LICENSE_KEY = key;
};
export const getLicenseText = (license) => {
    if (license === LICENSE_STATE.VALID)
        return 'forms.js license key is valid';
    if (license === LICENSE_STATE.OUTDATED)
        return 'forms.js license key is outdated';
    return 'forms.js license key is invalid';
};
export const handleInvalidLicenseLog = (license) => {
    console.error('***********************************************');
    if (license === LICENSE_STATE.INVALID) {
        console.error('forms.js license key is invalid, please use a valid license key');
    }
    else {
        console.error('forms.js license key is outdated, please update your license key');
    }
    console.error('***********************************************');
};
export const usesLicensedFetures = () => {
    return USES_LICENSED_FETURES;
};
export const useLicensedFetures = () => {
    USES_LICENSED_FETURES = true;
};
